from humbletray import layout, systray, chromeapp
import justpy as jp

wp = jp.WebPage(delete_flag=False)


def serve(q):
    print("serve")
    jp.Div(text="Success", a=wp)
    jp.justpy(lambda: wp)


def main_v1():

    app = chromeapp.ChromeApp("http://localhost:8000", "humbletray", (800, 600), lockPort=None, chromeargs=[])
    systray.run_gui(serve)
    print("closing")
    app.exit()
    try:
        app.wait()  # block
    except KeyboardInterrupt:
        print("-Process stopped")
    app.exit()


if __name__ == "__main__":
    main_v1()


import os

# Functions
# =========
#
# .. _get_hook_dirs:
#
# get_hook_dirs
# -------------
#
# Tell PyInstaller where to find hooks provided by this distribution;
# this is referenced by the :ref:`hook registration <hook_registration>`.
# This function returns a list containing only the path to this
# directory, which is the location of these hooks.


def get_hook_dirs():
    return [os.path.dirname(__file__)]


from pathlib import Path
from PyInstaller.utils.hooks import get_package_paths

# ok great, we're getting the templates directly from the horses mouth
templates = Path(get_package_paths("justpy.templates")[1])
leaf = Path(get_package_paths("humbletray")[1]) / "leaf.png"

datas = [(str(leaf), "."), (templates, "justpy/templates")]

hiddenimports = [
    "pystray._win32",
    "schedule",
    "uvicorn.lifespan.on",
    "uvicorn.protocols.websockets.auto",
    "uvicorn.protocols.http.auto",
    "uvicorn.logging",
    "uvicorn.loops.auto",
    "uvicorn.protocols",
]


from typing import List
from pystray import MenuItem, Menu
import pystray, sys, os
from PIL import Image
import time
from multiprocessing import Process, freeze_support, Queue
import justpy as jp
from humbletray import chromeapp

# from loguru import logger

q = Queue()

fig_name = "leaf.png"

# if getattr(sys, "frozen", False):
#     application_path = os.path.dirname(sys.executable)
# else:
#     try:
#         app_full_path = os.path.realpath(__file__)
#         application_path = os.path.dirname(app_full_path)
#     except NameError:
#         application_path = os.getcwd()

# fig_full_path = os.path.join(application_path, fig_name)
def resource_path(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.abspath(".")

    return os.path.join(base_path, relative_path)


fig_full_path = resource_path(fig_name)


class SystrayIconMenu:
    def __init__(self, icon, menu=[], exit=None, schedule=None):
        self.icon = pystray.Icon("mon")
        self.icon.title = "Tooltip"
        self.icon.icon = Image.open(icon)
        self.schedule = schedule
        menu.append(("Exit", lambda: self.exit()))

        menu_items = []
        for name, action in menu:
            menu_items.append(MenuItem(name, action))
        self.icon.menu = Menu(*menu_items)

        if exit:
            self.exit = exit

    def action(self):
        print("action")

    def exit(self):
        self.icon.visible = False
        self.icon.stop()

    def setup(self, icon):
        icon.visible = True

        i = 0
        while icon.visible:
            # Some payload code
            if self.schedule:
                self.schedule.run_pending()
            print(i)
            i += 1

            time.sleep(5)

    def run(self):
        self.icon.run(self.setup)


class SystrayApp(object):
    def __init__(self, start_server, menu, icon, schedule=None):
        self.start_server = start_server
        self.menu = menu
        self.fig = icon
        self.schedule = schedule

    def run(self):
        freeze_support()
        server = Process(target=self.start_server, args=(q,))
        server.daemon = True
        server.start()

        def action():
            print("gui action")
            import atexit

            def clean_exit():
                # logger.debug("clean exit")
                app.exit()

            atexit.register(clean_exit)
            app = chromeapp.ChromeApp("http://localhost:8000", "humbletray", (800, 600), lockPort=None, chromeargs=[])

        menu = [("Open App", action)]

        icon = SystrayIconMenu(self.fig, menu, schedule=self.schedule)
        icon.run()

        server.terminate()
        server.join(timeout=1.0)


def run_gui(start_server, menu=None, fig=fig_full_path, schedule=None):
    # Todo(Ksmith): add scheduler integration
    app = SystrayApp(start_server, menu, fig, schedule)
    app.run()



import sys, os, webbrowser
import shutil, logging
import tempfile, subprocess
from loguru import logger


class FULLSCREEN:
    pass


CHROMECACHE = ".cache"
size = (800, 600)


class ChromeApp:
    def __init__(self, url, appname="driver", size=None, lockPort=None, chromeargs=[]):
        def find_chrome_win():
            import winreg  # TODO: pip3 install winreg

            reg_path = r"SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\chrome.exe"
            for install_type in winreg.HKEY_CURRENT_USER, winreg.HKEY_LOCAL_MACHINE:
                try:
                    with winreg.OpenKey(install_type, reg_path, 0, winreg.KEY_READ) as reg_key:
                        return winreg.QueryValue(reg_key, None)
                except WindowsError:
                    pass

        def find_chrome_mac():
            default_dir = "/Applications/Google Chrome.app/Contents/MacOS/Google Chrome"
            if os.path.exists(default_dir):
                return default_dir

        if sys.platform[:3] == "win":
            exe = find_chrome_win()
        elif sys.platform == "darwin":
            exe = find_chrome_mac()
        else:
            for i in ["chromium-browser", "chromium", "google-chrome", "chrome"]:
                try:
                    exe = webbrowser.get(i).name
                    break
                except webbrowser.Error:
                    exe = None

        if not exe:
            raise Exception("no chrome browser, no app-mode !")
        else:
            args = [  # https://peter.sh/experiments/chromium-command-line-switches/
                exe,
                "--app=" + url,  # need to be a real http page !
                "--app-id=%s" % (appname),
                "--app-auto-launched",
                "--no-first-run",
                "--no-default-browser-check",
                "--disable-notifications",
                "--disable-features=TranslateUI",
                # ~ "--no-proxy-server",
            ] + chromeargs
            if size:
                if size == FULLSCREEN:
                    args.append("--start-fullscreen")
                else:
                    args.append("--window-size=%s,%s" % (size[0], size[1]))

            if lockPort:  # enable reusable cache folder (coz only one instance can be runned)
                self.cacheFolderToRemove = None
                args.append("--remote-debugging-port=%s" % lockPort)
                args.append("--disk-cache-dir=%s" % CHROMECACHE)
                args.append("--user-data-dir=%s/%s" % (CHROMECACHE, appname))
            else:
                self.cacheFolderToRemove = os.path.join(tempfile.gettempdir(), appname + "_" + str(os.getpid()))
                args.append("--user-data-dir=" + self.cacheFolderToRemove)
                args.append("--aggressive-cache-discard")
                args.append("--disable-cache")
                args.append("--disable-application-cache")
                args.append("--disable-offline-load-stale-cache")
                args.append("--disk-cache-size=0")

            logger.debug("CHROME APP-MODE: %s", " ".join(args))
            self._p = subprocess.Popen(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

    def wait(self):
        self._p.wait()

    def __del__(self):  # really important !
        print("deleting")
        self._p.kill()
        if self.cacheFolderToRemove:
            shutil.rmtree(self.cacheFolderToRemove, ignore_errors=True)

    def exit(self):
        print("exiting")
        # ~ self._com(dict(method="Browser.close"))
        self._p.kill()
        sys.exit()



import justpy as jp


def test_imports():
    from humbletray import main, systray

def test_leaf_icon():
    from humbletray import systray
    fig = systray.fig_full_path
    assert 'leaf.png' in fig

def test_quasar_layout():
    from humbletray import layout
    """Basic sanity checks."""
    default = layout.DefaultLayout()
    default.content.add(jp.Div(text="success"))
    assert 'success' in default.to_html()
    assert 'q-layout' in default.to_html()




#################### Maintained by Hatch ####################
# This file is auto-generated by hatch. If you'd like to customize this file
# please add your changes near the bottom marked for 'USER OVERRIDES'.
# EVERYTHING ELSE WILL BE OVERWRITTEN by hatch.
#############################################################
from io import open

from setuptools import find_packages, setup

with open("humbletray/__init__.py", "r") as f:
    for line in f:
        if line.startswith("__version__"):
            version = line.strip().split("=")[1].strip(" '\"")
            break
    else:
        version = "0.0.1"

with open("README.rst", "r", encoding="utf-8") as f:
    readme = f.read()

REQUIRES = []

kwargs = {
    "name": "humbletray",
    "version": version,
    "description": "",
    "long_description": readme,
    "author": "Kevin Smith",
    "author_email": "ksmith@99hats.com",
    "maintainer": "Kevin Smith",
    "maintainer_email": "ksmith@99hats.com",
    "url": "https://github.com/_/humbletray",
    "license": "MIT/Apache-2.0",
    "classifiers": [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    "install_requires": REQUIRES,
    "tests_require": ["coverage", "pytest"],
    "packages": find_packages(exclude=("tests", "tests.*")),
}

#################### BEGIN USER OVERRIDES ####################
# Add your customizations in this section.

kwargs["install_requires"] = ["schedule", "justpy", "pystray", "loguru"]
kwargs["include_package_data"] = True

###################### END USER OVERRIDES ####################

setup(**kwargs)



from contextlib import contextmanager
import justpy as jp


@contextmanager
def _(w):
    # to nest gui building
    yield w


class DefaultLayout(jp.QuasarPage):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        layout = jp.QLayout(view="hHh Lpr lff", classes="q-pa-md", style="height:300px", a=self)

        async def toggle_menu(comp, msg):
            print("button clicked")
            await drawer.run_method("toggle()", msg.websocket)

        with _(jp.QHeader(elevated=True, classes="bg-black", a=layout)) as header:
            with _(jp.QToolbar(a=header)) as toolbar:
                toolbar += jp.QBtn(flat=True, round=True, dense=True, icon="menu", click=toggle_menu)
                toolbar += jp.QToolbarTitle(text="Header!!")

        with _(
            jp.QDrawer(
                v_model="drawer",
                show_if_above=True,
                width=200,
                breakpoint=500,
                bordered=True,
                content_class="bg-grey-3",
                a=layout,
            )
        ) as drawer:
            with _(jp.QScrollArea(classes="fit", a=drawer)) as scrollarea:
                with _(jp.QList(a=scrollarea)) as qlist:
                    with _(jp.Div(a=qlist)) as menu:
                        self.menu = menu
        page_container = jp.QPageContainer(a=layout)
        self.content = jp.QPage(padding=True, a=page_container)



def main():
    """ Usage """

    def create_wp():
        """Create separate websession

        Otherwise, it's a shared session which of course can be cool.
        """
        wp = DefaultLayout()  # delete_flag=False)
        return wp

    def foo(self, msg):
        msg.page.content.delete_components()
        jp.QBtn(text="back to main", click=index, a=msg.page.content)
        for n in range(0, 155):
            msg.page.content.add(
                jp.P(
                    text="Lorem ipsum dolor sit amet consectetur adipisicing elit. Fugit nihil praesentium molestias a adipisci, dolore vitae odit, quidem consequatur optio voluptates asperiores pariatur eos numquam rerum delectus commodi perferendis voluptate?"
                )
            )

    def index(self, msg):
        msg.page.content.delete_components()
        jp.QBtn(text="foo", click=foo, a=msg.page.content)

    def create_menu(menu_list, wp3):
        wp3.menu.delete_components()
        for i, menu in enumerate(menu_list):
            qitem = jp.QItem(clickable=True, v_ripple=True, a=wp3.menu, click=menu["page"])
            qs1 = jp.QItemSection(avatar=True, a=qitem)
            qs1 += jp.QIcon(name=menu["icon"])
            qitem += jp.QItemSection(text=menu["label"])
            if menu["separator"]:
                wp3.menu += jp.QSeparator(key="sep" + str(i))

    def app():

        wp = create_wp()  # if you want separate web sessions
        menu_list = [
            {"icon": "inbox", "label": "Inbox", "separator": True, "page": foo},
            {"icon": "send", "label": "Outbox", "separator": False, "page": index},
            {"icon": "delete", "label": "Trash", "separator": False, "page": foo},
            {"icon": "error", "label": "Spam", "separator": True, "page": index},
            {
                "icon": "settings",
                "label": "Settings",
                "separator": False,
                "page": foo,
            },
            {
                "icon": "feedback",
                "label": "Send Feedback",
                "separator": False,
                "page": index,
            },
            {
                "icon": "help",
                "iconColor": "primary",
                "label": "Help",
                "separator": False,
                "page": foo,
            },
        ]
        create_menu(menu_list, wp)

        class msg(object):  # a weird bit since we're avoiding use of requests
            page = wp

        index(None, msg)
        return wp

    jp.justpy(app)
    # japp = jp.app
    # jp.justpy(app, start_server=False)
    # from starlette.testclient import TestClient
    # from starlette.testclient import TestClient

    # def test_app():
    #     client = TestClient(japp)
    #     response = client.get("/")
    #     assert response.status_code == 200
    #     print(response.status_code)
    #     import pdb

    #     pdb.set_trace()

    # test_app()

if __name__ == "__main__":
    main()